/*
   This file is part of the index package,
   Copyright (C) 1997 Harold Tay, haroldtay@pacific.net.sg

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA
 */
#ifndef HASH_H
#define HASH_H

/* Hash table routines.  Only pointers to the data (keys as well
   as values) are stored, so the user must make sure the object being
   stored is allocated, and remains allocated for the duration of
   storage.  The user can alter the pointed-to object with impunity.

   The routines treat each key and value as a raw pointer, so they
   don't have to be type char *.
 */

#ifndef HASH_MAXTABLES
/* Max number of tables you can use */
#define HASH_MAXTABLES 8
#endif

/* A hashing function by djb */
extern unsigned int string_hash_function(char * text);

/* If hash is NULL, string_hash_function is used.  If compare is NULL,
   strcmp is used. compare is used to test for -equality- only. */
extern int hash_create(unsigned int (*hash)(char *),
                       int (*compare)(const char *, const char *), int hint);

/* Get the approx. size of the hash table */
extern int hash_size(int ti);

extern char * hash_fetch(int ti, char * key);

/* Insert; fail if exist */
extern int hash_insert(int ti, char * key, char * val);

/* Replace; fail if not exist */
extern char * hash_replace(int ti, char * key, char * val);

/* Remove entry */
extern int hash_delete(int ti, char * key, char ** keyp, char ** valp);

/* Destroy this hash table */
extern int hash_destroy(int ti);

/* If f returns non-zero, hash_walk immediately
   returns with that value. */
extern int hash_walk(int ti, int (*f)(char * key, char * val));

#endif
